/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016-2017 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __USB_PD_POLICY_ENGINE_H__
#define __USB_PD_POLICY_ENGINE_H__

#include "tusb422_common.h"
#include "tcpci.h"
#include "tcpm.h"

//#define ENABLE_VDM_SUPPORT
//#define ENABLE_DP_ALT_MODE_SUPPORT  /* Requires ENABLE_VDM_SUPPORT */
//#define ENABLE_HDMI_ALT_MODE_SUPPORT  /* Requires ENABLE_VDM_SUPPORT */
//#define ENABLE_HDMI_HDCP_SUPPORT  /* Requires ENABLE_HDMI_ALT_MODE_SUPPORT */

//#define ENABLE_USB_PD_REV3_SUPPORT
//#define ENABLE_EXTENDED_MSG_SUPPORT /* Requires ENABLE_USB_PD_REV3_SUPPORT */

#define MSG_BUF_SIZE    32  /* 32-bytes */
#define MAX_EXT_MSG_LEN 260 /* 260-bytes */

#define MAX_SOP_NUM 5     /* SOP, SOP', SOP", SOP_DBG', SOP_DBG" */

#define PD_STATE_HISTORY_LEN  16
#define PD_STATE_INDEX_MASK   0xF  /* bitmask based on history length */

#define HPD_IN_QUEUE_SIZE  4

typedef enum
{
	PE_UNATTACHED = 0,

	/* Source */
	PE_SRC_STARTUP,
	PE_SRC_DISCOVERY,
	PE_SRC_SEND_CAPS,
	PE_SRC_NEGOTIATE_CAPABILITY,
	PE_SRC_TRANSITION_SUPPLY,
	PE_SRC_TRANSITION_SUPPLY_EXIT,
	PE_SRC_READY,
	PE_SRC_DISABLED,
	PE_SRC_CAPABILITY_RESPONSE,
	PE_SRC_HARD_RESET,
	PE_SRC_HARD_RESET_RECEIVED,
	PE_SRC_TRANSITION_TO_DEFAULT,
	PE_SRC_GET_SINK_CAP,
	PE_SRC_WAIT_NEW_CAPS,
	PE_SRC_SEND_SOFT_RESET,
	PE_SRC_SOFT_RESET,
	PE_SRC_PING,
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	PE_SRC_SEND_SOURCE_ALERT,
	PE_SRC_SINK_ALERT_RECEIVED,
	PE_SRC_GIVE_SOURCE_CAP_EXT,
	PE_SRC_GIVE_SOURCE_STATUS,
	PE_SRC_GET_SINK_STATUS,
#endif

	/* Sink */
	PE_SNK_STARTUP,
	PE_SNK_DISCOVERY,
	PE_SNK_WAIT_FOR_CAPS,
	PE_SNK_EVALUATE_CAPABILITY,
	PE_SNK_SELECT_CAPABILITY,
	PE_SNK_TRANSITION_SINK,
	PE_SNK_READY,
	PE_SNK_HARD_RESET,
	PE_SNK_TRANSITION_TO_DEFAULT,
	PE_SNK_GIVE_SINK_CAP,
	PE_SNK_GET_SOURCE_CAP,
	PE_SNK_SEND_SOFT_RESET,
	PE_SNK_SOFT_RESET,
#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	PE_SNK_SOURCE_ALERT_RECEIVED,
	PE_SNK_SEND_SINK_ALERT,
	PE_SNK_GET_SOURCE_CAP_EXT,
	PE_SNK_GET_SOURCE_STATUS,
	PE_SNK_GIVE_SINK_STATUS,
#endif

	/* Sink or Source */
	PE_SEND_REJECT,
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	PE_SEND_NOT_SUPPORTED,
#endif

	/* Dual-Role */
	PE_DR_SRC_GIVE_SINK_CAP,
	PE_DR_SNK_GIVE_SOURCE_CAP,
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	PE_DR_SRC_GET_SOURCE_CAP_EXT,
	PE_DR_SNK_GIVE_SOURCE_CAP_EXT,
#endif

	/* BIST */
	PE_BIST_CARRIER_MODE,
	PE_BIST_TEST_MODE,

	/* Error Recovery */
	PE_ERROR_RECOVERY,

	/* Data Role Swap */
	PE_DRS_SEND_SWAP,
	PE_DRS_EVALUATE_SWAP,
	PE_DRS_ACCEPT_SWAP,
	PE_DRS_CHANGE_ROLE,

	/* Power Role Swap */
	PE_PRS_SEND_SWAP,
	PE_PRS_EVALUATE_SWAP,
	PE_PRS_WAIT_SWAP,
	PE_PRS_ACCEPT_SWAP,
	PE_PRS_TRANSITION_TO_OFF,
	PE_PRS_ASSERT_RD,
	PE_PRS_WAIT_SOURCE_ON,
	PE_PRS_ASSERT_RP,
	PE_PRS_SOURCE_ON,
	PE_PRS_SOURCE_ON_EXIT,

	/* VCONN Swap */
	PE_VCS_SEND_SWAP,
	PE_VCS_EVALUATE_SWAP,
	PE_VCS_ACCEPT_SWAP,
	PE_VCS_WAIT_FOR_VCONN,
	PE_VCS_TURN_OFF_VCONN,
	PE_VCS_TURN_ON_VCONN,
	PE_VCS_SEND_PS_RDY,

	/* VDM */
#ifdef ENABLE_VDM_SUPPORT
	PE_SRC_VDM_IDENTITY_REQUEST,  /* request to cable before explicit contact */
	PE_INIT_PORT_VDM_IDENTITY_REQUEST, /* request to port partner */
	PE_INIT_VDM_SVIDS_REQUEST,
	PE_INIT_VDM_MODES_REQUEST,
	PE_INIT_VDM_ATTENTION_REQUEST,
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	PE_INIT_VDM_DP_STATUS_UPDATE, /* DFP_U only */
	PE_INIT_VDM_DP_CONFIG,        /* DFP_U only */
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	PE_INIT_VDM_HDMI_STATUS_UPDATE,
//	PE_INIT_VDM_HDMI_CEC_WRITE,
//	PE_INIT_VDM_HDMI_CEC_READ,
	PE_INIT_VDM_HDMI_DDC_WRITE,
	PE_INIT_VDM_HDMI_DDC_READ,
#endif

	PE_RESP_VDM_NAK,
	PE_RESP_VDM_GET_IDENTITY,
	PE_RESP_VDM_GET_SVIDS,
	PE_RESP_VDM_GET_MODES,
	PE_RCV_VDM_ATTENTION_REQUEST,
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	PE_RESP_VDM_DP_STATUS_UPDATE,
	PE_RESP_VDM_DP_CONFIG,
#endif
#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
	PE_RESP_VDM_HDMI_STATUS_UPDATE,
//	PE_RESP_VDM_HDMI_CEC_WRITE,
//	PE_RESP_VDM_HDMI_CEC_READ,
	PE_RESP_VDM_HDMI_DDC_WRITE,
	PE_RESP_VDM_HDMI_DDC_READ,
#endif

	PE_DFP_VDM_MODE_ENTRY_REQUEST,
	PE_DFP_VDM_MODE_EXIT_REQUEST,

	PE_UFP_VDM_EVAL_MODE_ENTRY,
	PE_UFP_VDM_MODE_EXIT,
#endif

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	PE_GET_BATTERY_CAP,
	PE_GIVE_BATTERY_CAP,
	PE_GET_BATTERY_STATUS,
	PE_GIVE_BATTERY_STATUS,
	PE_GET_MANUFACTURER_INFO,
	PE_GIVE_MANUFACTURER_INFO,
	PE_SEND_SECURITY_REQUEST,
	PE_SEND_SECURITY_RESPONSE,
	PE_SECURITY_RESP_RECEIVED,
	PE_SEND_FW_UPDATE_REQUEST,
	PE_SEND_FW_UPDATE_RESPONSE,
	PE_FW_UPDATE_RESP_RECEIVED,
#endif

	PE_NUM_STATES
} usb_pd_pe_state_t;

extern const char * const pdstate2string[PE_NUM_STATES];

typedef enum
{
	NO_ALT_MODE = 0,
	DP_ALT_MODE = (1 << 0),
	TBOLT_ALT_MODE = (1 << 1),
	HDMI_ALT_MODE = (1 << 2)
} alt_mode_t;

typedef enum
{
	CONFIG_FOR_USB = 0,
	CONFIG_UFP_U_AS_DFP_D = 1,
	CONFIG_UFP_U_AS_UFP_D = 2,
	CONFIG_MASK = 0x3
} dp_config_sel_t;

typedef enum
{
	NOTIFY_HPD_NONE = 0,
	NOTIFY_HPD_LOW  = 1,
	NOTIFY_HPD_HIGH = 2,
	NOTIFY_HPD_IRQ  = 3
} hpd_notify_t;

//typedef enum
//{
//    // BQ - maybe these can be bool in struct below.
//    PD_FLAGS_DUAL_ROLE_POWER = (0 << 0), 
//    PD_FLAGS_DUAL_ROLE_DATA  = (1 << 1), 
//    PD_FLAG_USB_COMM_CAPABLE = (1 << 2),
//} pd_flags_t;


#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
typedef enum
{
	RCH_WAIT_FOR_MSG = 0,
	RCH_REQUESTING_CHUNK,
	RCH_WAITING_CHUNK,
	NUM_RCH_STATES
} rch_state_t;

typedef enum
{
	TCH_WAIT_FOR_MSG_REQ = 0,
	TCH_CONSTRUCT_MSG,
	TCH_SENDING_MSG,
	TCH_WAIT_CHUNK_REQ,
	NUM_TCH_STATES
} tch_state_t;

#endif


typedef struct
{
	uint8_t             msg_buf[MSG_BUF_SIZE];

#if defined ENABLE_USB_PD_REV3_SUPPORT && defined ENABLE_EXTENDED_MSG_SUPPORT
	uint8_t             extended_msg_buf[MAX_EXT_MSG_LEN];
	uint16_t            extended_msg_byte_cnt;
	uint8_t             expected_chunk_num;
	uint8_t             chunk_num_to_send;
	uint8_t             ndos_to_send;
	bool                abort_flag;
	rch_state_t         rch_state; /* chunked Rx state */
	tch_state_t         tch_state; /* chunked Tx state */
	bool                chunked_state_change;
	uint8_t             chunked_msg_type;
#endif

	usb_pd_pe_state_t   state[PD_STATE_HISTORY_LEN];
	unsigned int        state_idx;
	usb_pd_pe_state_t   *current_state;
	bool                state_change;

	tcpc_device_t       *tcpc_dev;

//    uint8_t             flags;   /* BQ can convert all bools to single flag to save RAM if needed */
	unsigned int        port;
	struct tusb422_timer_t  timer;
	struct tusb422_timer_t  timer2; /* LFO timer */

	uint8_t             p2p_spec_rev; /* Port to port operating spec rev */
	uint8_t             p2c_spec_rev; /* Port to cable plug operating spec rev */
	uint8_t             power_role;
	uint8_t             data_role;

	tcpc_transmit_t     tx_sop_type;		   /* For incrementing correct msg ID when Tx is successful */
	uint8_t             msg_id[MAX_SOP_NUM];   /* For Tx. Masked off to 3-bits when building msg header */

	uint8_t             stored_msg_id[MAX_SOP_NUM];	  /* For Rx */
	uint8_t             rx_msg_data_len;
	uint8_t             rx_msg_type;
#ifdef ENABLE_USB_PD_REV3_SUPPORT
	uint8_t             rx_msg_extended;  /* PD 3.0 */
#endif
	uint8_t             rx_msg_spec_rev;
	uint8_t             rx_sop;

	uint8_t             hard_reset_cnt;
	uint8_t             caps_cnt;
	bool                vconn_source;

	bool                non_interruptable_ams;
	bool                swap_source_start;
	bool                start_src_no_response_timer;
	bool                pd_connected;  /* PD connected after a PD msg and GoodCRC have been exchanged */
	bool                pd_connected_since_attach;	/* PD connected at any point since attachment */
	bool                explicit_contract;
	bool                vbus_present;
	bool                power_role_swap_in_progress;
	bool                hard_reset_complete;

	uint8_t             object_position;  /* Range: 1 - 7 */
	uint8_t             selected_snk_pdo_idx;
	uint32_t            rdo;
	uint32_t            selected_pdo;
	uint32_t            prev_selected_pdo;

	bool                request_goto_min;
	bool                snk_goto_min;
	bool                snk_wait;

	uint32_t            src_pdo[PD_MAX_PDO_NUM];
	uint32_t            snk_pdo[PD_MAX_PDO_NUM];

	uint16_t            cable_max_current_ma;
	uint16_t            cable_max_voltage_mv; 
	bool                active_cable;

	uint8_t             active_alt_modes;

	bool                cable_plug_comm_required;
	bool                get_sink_caps_complete;

	bool                error_recovery_pending;
#ifdef ENABLE_VDM_SUPPORT
	bool                send_cable_soft_reset;
	bool                cable_id_req_complete;
	uint8_t             discover_identity_cnt;	 // When sending Discover Identity to cable plug.  Max 20.  Zero upon data role swap.
	uint8_t             vdm_busy_cnt;  // For UFP or Cable Plug.  Max of 1 responder busy response.  Reset upon non-busy response.
	bool                send_src_cap_start_once;
	bool                send_src_cap_timeout;

	bool                port_partner_id_req_complete;
	uint8_t             p2p_vdm_version; /* Port to port structured VDM version */
	uint8_t             p2c_vdm_version; /* Port to cable plug structured VDM version */
	uint32_t            vdm_buf[7]; /* VDM header followed by 6 VDOs */
	bool                vdm_in_progress;
	bool                vdm_initiator;
	uint8_t             svid_index;

	uint8_t             matched_alt_modes;
	bool                vdm_pending;
	usb_pd_pe_state_t   vdm_pending_state;
	bool                mode_exit_pending;
	bool                attention_pending;
#ifdef ENABLE_DP_ALT_MODE_SUPPORT
	uint32_t            displayport_remote_caps; /* from port partner */
	uint8_t             displayport_matched_pin_assignments;
	uint8_t             displayport_selected_pin_assignment;

	bool                displayport_alt_mode_configured;
	dp_config_sel_t     displayport_config;
	uint8_t             displayport_lane_cnt;
	uint32_t            displayport_status;  /* local port status */
	uint32_t            displayport_remote_status;  /* remote port status */

	bool                hpd_in_changed;
	uint8_t             hpd_in_value;
	hpd_notify_t        hpd_in_queue[HPD_IN_QUEUE_SIZE];   /* Worst case: HPD_Low, HPD_High, IRQ_HPD, IRQ_HPD. */
	uint8_t             hpd_in_queue_idx;
	uint8_t             hpd_in_send_idx;
#ifdef DP_DONGLE
	struct tusb422_timer_t  dp_timer;
	uint8_t             hpd_in_previous_value;
	bool                redriver_configured;
#endif
#endif /* ENABLE_DP_ALT_MODE_SUPPORT */

#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
#define NUM_EDID_BLOCKS  8  /* Most EDID have only 2 blocks */
	uint8_t             ddc_slave_addr;
	uint8_t             ddc_reg_offset;
	uint8_t             ddc_reg_size;
	uint8_t             ddc_sequence_num;
	uint8_t             edid_ext_blocks;
	uint8_t             hdmi_edid[NUM_EDID_BLOCKS * 128]; /* each block is 128-bytes */
#ifdef HDMI_DONGLE
	struct tusb422_timer_t  hdmi_timer;
	bool                hpd_in_changed;
	uint8_t             hpd_in_value;
	uint8_t             hpd_in_previous_value;
	bool                hdmi_initial_status_update_complete;
#endif
#ifdef ENABLE_HDMI_HDCP_SUPPORT
#define HDCP_BUF_SIZE  1024 /* max msg size per HDCP on HDMI spec */
	uint8_t             hdcp_buf[HDCP_BUF_SIZE];
	uint16_t            hdcp_byte_cnt;
#endif
	uint32_t            hdmi_status;  /* local status if we are cable adapter, remote if we are source */
#endif

#endif /* ENABLE_VDM_SUPPORT */

	bool                externally_powered;
	bool                remote_externally_powered;

#ifdef ENABLE_USB_PD_REV3_SUPPORT
	bool                fast_role_swap_supported;
	bool                sink_tx_ok;
#endif
} usb_pd_port_t;

typedef enum
{
	STATUS_OK = 0,
	STATUS_REQUEST_NOT_SUPPORTED_IN_CURRENT_STATE
} usb_pd_pe_status_t;


typedef enum
{
	PRL_ALERT_MSG_RECEIVED = 0,
	PRL_ALERT_MSG_TX_SUCCESS,
	PRL_ALERT_MSG_TX_DISCARDED,
	PRL_ALERT_MSG_TX_FAILED,
	PRL_ALERT_HARD_RESET_RECEIVED
} usb_pd_prl_alert_t;

typedef enum
{
	PD_POLICY_MNGR_REQ_GET_SINK_CAPS,
	PD_POLICY_MNGR_REQ_SRC_CAPS_CHANGE,
	PD_POLICY_MNGR_REQ_GOTO_MIN,
	PD_POLICY_MNGR_REQ_UPDATE_REMOTE_CAPS,
	PD_POLICY_MNGR_REQ_PR_SWAP,
	PD_POLICY_MNGR_REQ_DR_SWAP,
	PD_POLICY_MNGR_REQ_VCONN_SWAP,
	PD_POLICY_MNGR_REQ_HARD_RESET,

	PD_POLICY_MNGR_REQ_STOP
} pd_policy_manager_request_t;

#ifdef ENABLE_DP_ALT_MODE_SUPPORT
typedef enum
{
	IRQ_HPD             = (1 << 8),
	HPD_STATE_HIGH      = (1 << 7),
	REQ_DP_MODE_EXIT    = (1 << 6),
	REQ_USB_CONFIG      = (1 << 5),
	MULTIFUNC_PREFERRED = (1 << 4),
	ADAPTER_ENABLED     = (1 << 3),
	ADAPTER_POWER_LOW   = (1 << 2),
	UFP_D_CONNECTED     = (1 << 1),
	DFP_D_CONNECTED     = 1
} display_port_status_t;

void usb_pd_pe_update_dp_status(unsigned int port, uint32_t clr_bits, uint32_t set_bits);
#endif

#ifdef ENABLE_HDMI_ALT_MODE_SUPPORT
typedef enum
{
	HDCP_PRIMARY_SLAVE_ADDR   = 0x74,
	HDCP_SECONDARY_SLAVE_ADDR = 0x76,
	EDID_SLAVE_ADDR           = 0xA0,
	EDDC_SEGMENT_POINTER_ADDR = 0x60
} hdmi_slave_addr_t;

typedef enum
{
	CEC_LOGICAL_ADDR_LIST_MASK   = 0x7FFF8,
	CEC_AVAILABLE   = (1 << 2),
	EDID_AVAILABLE  = (1 << 1),
	HPD_STATUS      = 1
} hdmi_status_t;

#endif

void usb_pd_pe_notify(unsigned int port, usb_pd_prl_alert_t prl_alert);
void usb_pd_pe_connection_state_change_handler(unsigned int port, tcpc_state_t state);
void usb_pd_pe_voltage_alarm_handler(unsigned int port, bool hi_voltage);
void usb_pd_pe_current_change_handler(unsigned int port, tcpc_cc_snk_state_t curr_adv);

int usb_pd_policy_manager_request(unsigned int port, pd_policy_manager_request_t req);
bool usb_pd_pe_is_remote_externally_powered(unsigned int port);
usb_pd_port_t * usb_pd_pe_get_device(unsigned int port);

void usb_pd_pe_init(unsigned int port, usb_pd_port_config_t *config);

#endif
