/*
 * adc.c
 *
 *  Created on: Aug 8, 2018
 *      Author: a0220433
 */

#include <msp430.h>
#include <adc12_b.h>
#include "gpio.h"
#include "pb.h"
#include "ref_a.h"
#include "string.h"

#ifdef USE_MSP430_ADC_IBAT_IBUS
#define IBUS_ADC_IDX    ADC12_B_MEMORY_0
#define IBAT_ADC_IDX    ADC12_B_MEMORY_1
#endif

#define VBAT_ADC_IDX    ADC12_B_MEMORY_2
#define IOA_ADC_IDX     ADC12_B_MEMORY_4

extern power_bank_t pb;
extern struct pb_config pb_config;

void adc_init()
{
    ADC12_B_initParam initParam = {0};
    ADC12_B_configureMemoryParam cfgMemParam = {0};

 //   Ref_A_setReferenceVoltage(REF_A_BASE, REF_A_VREF2_5V);
//    Ref_A_enableReferenceVoltage(REF_A_BASE);


    initParam.clockSourceSelect = ADC12_B_CLOCKSOURCE_SMCLK;
    initParam.clockSourceDivider = ADC12_B_CLOCKDIVIDER_4;
    ADC12_B_init(ADC12_B_BASE, &initParam);

    //setup as analog input pin
#ifdef USE_MSP430_ADC_IBAT_IBUS
    //A0: IBus
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1, GPIO_PIN0, GPIO_TERNARY_MODULE_FUNCTION);
    //A1: IBat
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1, GPIO_PIN1, GPIO_TERNARY_MODULE_FUNCTION);
#endif
    //A2: VBat
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1, GPIO_PIN2, GPIO_TERNARY_MODULE_FUNCTION);

    //A4: Io_A
    GPIO_setAsPeripheralModuleFunctionInputPin(GPIO_PORT_P1, GPIO_PIN4, GPIO_TERNARY_MODULE_FUNCTION);

    cfgMemParam.refVoltageSourceSelect = ADC12_B_VREFPOS_INTBUF_VREFNEG_VSS;

#ifdef USE_MSP430_ADC_IBAT_IBUS
    cfgMemParam.memoryBufferControlIndex = IBUS_ADC_IDX;
    cfgMemParam.inputSourceSelect = ADC12_B_INPUT_A0;
    ADC12_B_configureMemory(ADC12_B_BASE, &cfgMemParam);

    cfgMemParam.memoryBufferControlIndex = IBAT_ADC_IDX;
    cfgMemParam.inputSourceSelect = ADC12_B_INPUT_A1;
    ADC12_B_configureMemory(ADC12_B_BASE, &cfgMemParam);
#endif

    cfgMemParam.memoryBufferControlIndex = VBAT_ADC_IDX;
    cfgMemParam.inputSourceSelect = ADC12_B_INPUT_A2;
    ADC12_B_configureMemory(ADC12_B_BASE, &cfgMemParam);

    cfgMemParam.memoryBufferControlIndex = IOA_ADC_IDX;
    cfgMemParam.inputSourceSelect = ADC12_B_INPUT_A4;

    cfgMemParam.endOfSequence = ADC12_B_ENDOFSEQUENCE;

    ADC12_B_configureMemory(ADC12_B_BASE, &cfgMemParam);

    ADC12_B_setupSamplingTimer(ADC12_B_BASE, ADC12_B_CYCLEHOLD_4_CYCLES, ADC12_B_CYCLEHOLD_4_CYCLES, ADC12_B_MULTIPLESAMPLESENABLE);

    ADC12_B_setResolution(ADC12_B_BASE, ADC12_B_RESOLUTION_12BIT);
    ADC12_B_setDataReadBackFormat(ADC12_B_BASE, ADC12_B_UNSIGNED_BINARY);
  //  ADC12_B_enable(ADC12_B_BASE);

}

#define ADC_REF     2500
#define ADC12_LSB   0.6103515625//(ADC_REF / 4096)
#define AVG_CNT     5
void adc_update_results()
{
    int32_t res;
    int i;
    int32_t bus_curr, bat_curr, ioA_curr, bat_volt;

    Ref_A_setReferenceVoltage(REF_A_BASE, REF_A_VREF2_5V);
    Ref_A_enableReferenceVoltage(REF_A_BASE);
    __delay_cycles(5000);
    ADC12_B_enable(ADC12_B_BASE);
    __delay_cycles(1000);

    bus_curr = bat_curr = ioA_curr = bat_volt = 0;
    for (i = 0; i < AVG_CNT; i++) {
    #ifdef USE_MSP430_ADC_IBAT_IBUS
        ADC12_B_startConversion(ADC12_B_BASE, ADC12_B_START_AT_ADC12MEM0, ADC12_B_SEQOFCHANNELS);
    #else
        ADC12_B_startConversion(ADC12_B_BASE, ADC12_B_START_AT_ADC12MEM2, ADC12_B_SEQOFCHANNELS);
    #endif

        while(ADC12_B_isBusy(ADC12_B_BASE));

    #ifdef USE_MSP430_ADC_IBAT_IBUS
        res = (int16_t)ADC12_B_getResults(ADC12_B_BASE, IBUS_ADC_IDX);
    //    res >>= 4;
        if (pb.ibus_gain)
            //pb.bus_curr = (res * ADC12_LSB) * 1000 /pb_config.bus_rsense /pb.ibus_gain;
            bus_curr += res * 61 /pb.ibus_gain;

        res = (int16_t)ADC12_B_getResults(ADC12_B_BASE, IBAT_ADC_IDX);
    //    res >>= 4;
        if (pb.ibat_gain)
            bat_curr += (res * ADC12_LSB) * 1000 /pb_config.bat_rsense /pb.ibat_gain;
           // pb.bat_curr = res * 61 /pb.ibat_gain;
    #endif

        res = ADC12_B_getResults(ADC12_B_BASE, IOA_ADC_IDX);
        ioA_curr += res * ADC12_LSB * 20/*1000/50*/;//

        res = ADC12_B_getResults(ADC12_B_BASE, VBAT_ADC_IDX);
        bat_volt += res * ADC12_LSB * 3.342/0.332;
    }


    ADC12_B_disable(ADC12_B_BASE);
    Ref_A_disableReferenceVoltage(REF_A_BASE);


    pb.bus_curr = bus_curr / AVG_CNT;
    pb.bat_curr = bat_curr / AVG_CNT;
    pb.ioA_curr = ioA_curr / AVG_CNT;
    pb.bat_volt = bat_volt / AVG_CNT;


}
