/*
 * main.c
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "msp430.h"
#include "driverlib.h"
#include "tcpci.h"
#include "tcpm.h"
#include "tcpm_hal.h"
#include "timer.h"
#include "tusb422.h"
#include "usb_pd_policy_engine.h"
#include "tusb422_common.h"
#include <stdint.h>
#include "pb.h"
#include "led.h"
#include "volt_rsoc.h"
#include "bq25713.h"
#include "adc.h"

#if defined(ccs) && !defined(__ISB)
__attribute__( ( always_inline ) ) static inline void __ISB(void)
{
    __asm("  isb");
}
#endif

//#define DP_ALT_MODE_HOST
//#define HDMI_ALT_MODE_HOST

//#if defined(ccs)
//#include "cmsis_ccs.h"
//#endif

//#if DEBUG_LEVEL >= 1
extern volatile bool bRegDump;
extern volatile bool bErrorRecovery;
extern volatile bool bTryRoleSwap;
extern volatile bool bSendHardReset;
extern volatile bool bPRSwap;
extern volatile bool bDRSwap;
extern volatile bool bVCONNSwap;
//#endif

extern unsigned int g_alert_event;
extern unsigned int g_timer_expired;
extern timer_callback_t g_timer_callback[NUM_TCPC_DEVICES];

extern usb_pd_port_t pd[NUM_TCPC_DEVICES];

#define PORT0  0

#define PDO1   0
#define PDO2   1
#define PDO3   2
#define PDO4   3
#define PDO5   4
#define PDO6   5
#define PDO7   6

#define USB_VID 0x0451
#define USB_PID 0x0422
#define USB_BCD_DEVICE 0x0  /* device-defined revision number */



static void periodic_task(unsigned int port);

void pb_src_vbus_enable_cb(int mv, int ma);

#if defined HDMI_DONGLE || defined DP_DONGLE

const tcpc_config_t typec_config[NUM_TCPC_DEVICES] =
{
    [PORT0].intf = SMBUS_MASTER0,
    [PORT0].slave_addr = TUSB422_SLAVE_ADDR_NC,
    [PORT0].flags = 0,
    [PORT0].role = ROLE_SNK,
    [PORT0].rp_val = RP_DEFAULT_CURRENT,
};

const usb_pd_port_config_t pd_config[NUM_TCPC_DEVICES] =
{
    [PORT0].usb_comm_capable = true,
    [PORT0].usb_suspend_supported = false,
    [PORT0].externally_powered = false,
    [PORT0].dual_role_data = false,
    [PORT0].unchunked_msg_support = false, /* Always false for TUSB422 */

    [PORT0].num_src_pdos = 0,

    [PORT0].num_snk_pdos = 1,
    /* vSafe5V Sink PDO */
    [PORT0].snk_caps[PDO1].SupplyType = SUPPLY_TYPE_FIXED, 
    [PORT0].snk_caps[PDO1].MinV = PDO_VOLT(5000),
    [PORT0].snk_caps[PDO1].MaxV = 0, /* N/A */
    [PORT0].snk_caps[PDO1].MaxOperatingCurrent = PDO_CURR(500),
    [PORT0].snk_caps[PDO1].MinOperatingCurrent = PDO_CURR(100),
    [PORT0].snk_caps[PDO1].OperationalCurrent = PDO_CURR(500),
    [PORT0].snk_caps[PDO1].MaxOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO1].MinOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO1].OperationalPower = 0,  /* N/A */

    [PORT0].higher_capability = false,
    [PORT0].giveback_flag = false,
    [PORT0].no_usb_suspend = true,
    [PORT0].fast_role_swap_support = FR_SWAP_NOT_SUPPORTED,
    [PORT0].pdo_priority = PRIORITY_VOLTAGE,
	[PORT0].vconn_source_capable = true,

    [PORT0].auto_accept_swap_to_dfp = false,
    [PORT0].auto_accept_swap_to_ufp = false,
    [PORT0].auto_accept_swap_to_source = false,
    [PORT0].auto_accept_swap_to_sink = false,
    [PORT0].auto_accept_vconn_swap = false,

//	[PORT0].attempt_discover_identity = true,
    [PORT0].multi_function_preferred = false,

    [PORT0].ufp_alt_mode_entry_timeout_enable = true,
    [PORT0].id_header_vdo = (0x6C000000 + USB_VID), /* Alt-mode adapter */
    [PORT0].cert_stat_vdo = 0,
    [PORT0].product_vdo = ((uint32_t)USB_PID << 16) | USB_BCD_DEVICE,
    [PORT0].num_product_type_vdos = 1,   /* Set to 1 for Alt-mode adapter*/
    [PORT0].product_type_vdos[0] = 0x8,  /* 0x8 - VBUS req'd. Alt-mode adapter */
    [PORT0].product_type_vdos[1] = 0,
    [PORT0].product_type_vdos[2] = 0,

    [PORT0].num_svids = 1,
#ifdef HDMI_DONGLE
    [PORT0].svids[0] = 0xFF04, /* 0xFF04 = HDMI */
    [PORT0].modes[0] = 0x1,  /* 0x01 = Cable adapter HDMI 1.4b */
#else /* DP */
    [PORT0].svids[0] = 0xFF01, /* 0xFF01 = DisplayPort */
    // Most monitors cannot support 2-lane DP, so we only advertise pin assignment C.
    [PORT0].modes[0] = 0x405,  /* 0x405 = UFP_D, DP_v1.3, plug, USB 2 signaling may be required, UFP_D pin assignment C (4-lane DP) */
#endif
    [PORT0].svids[1] = 0,
    [PORT0].svids[2] = 0,
    [PORT0].modes[1] = 0,
    [PORT0].modes[2] = 0,
};

#else /*************************************************************************/


const tcpc_config_t typec_config[NUM_TCPC_DEVICES] =
{
    [PORT0].intf = SMBUS_MASTER0,
    [PORT0].slave_addr = TUSB422_SLAVE_ADDR_NC,
    [PORT0].flags = 0,//TC_FLAGS_TRY_SNK,
    [PORT0].role = ROLE_DRP,
    [PORT0].rp_val = /*RP_MEDIUM_CURRENT,//*/RP_HIGH_CURRENT,//RP_DEFAULT_CURRENT,
};

const usb_pd_port_config_t pd_config[NUM_TCPC_DEVICES] =
{
    [PORT0].usb_comm_capable = true,
    [PORT0].usb_suspend_supported = false,
    [PORT0].externally_powered = true,
    [PORT0].dual_role_data = true,
    [PORT0].unchunked_msg_support = false, /* Always false for TUSB422 */

    [PORT0].num_src_pdos = 7,
    /* vSafe5V Source PDO */
    [PORT0].src_caps[PDO1].SupplyType = SUPPLY_TYPE_FIXED,
    [PORT0].src_caps[PDO1].PeakI = PEAK_CURRENT_0,
    [PORT0].src_caps[PDO1].MinV = PDO_VOLT(5000),
    [PORT0].src_caps[PDO1].MaxV = 0,   /* N/A */  
    [PORT0].src_caps[PDO1].MaxI = PDO_CURR(3000), /* TUSB422 Rev.A EVM is limited to 2A unless R123 is changed to 80.6K-ohm */
    [PORT0].src_caps[PDO1].MaxPower = 0, /* N/A */ 
    /* Fixed, Battery, Variable, and Programmable Supply PDOs each in ascending voltage order */
    /* Standard PDOs are 5V, 9V, 15V, and 20V. All PDOs lower than the highest voltage
     * supported must be present. 5V@3A must be supported if supporting > 15W */
    [PORT0].src_caps[PDO2].SupplyType = SUPPLY_TYPE_FIXED,
    [PORT0].src_caps[PDO2].PeakI = PEAK_CURRENT_0,
    [PORT0].src_caps[PDO2].MinV = PDO_VOLT(9000),
    [PORT0].src_caps[PDO2].MaxV = 0,   /* N/A */  
    [PORT0].src_caps[PDO2].MaxI = PDO_CURR(3000),
    [PORT0].src_caps[PDO2].MaxPower = 0, /* N/A */ 

    [PORT0].src_caps[PDO3].SupplyType = SUPPLY_TYPE_FIXED,
    [PORT0].src_caps[PDO3].PeakI = PEAK_CURRENT_0,
    [PORT0].src_caps[PDO3].MinV = PDO_VOLT(15000),
    [PORT0].src_caps[PDO3].MaxV = 0,   /* N/A */
    [PORT0].src_caps[PDO3].MaxI = PDO_CURR(3000),
    [PORT0].src_caps[PDO3].MaxPower = 0, /* N/A */

    [PORT0].src_caps[PDO4].SupplyType = SUPPLY_TYPE_FIXED,
    [PORT0].src_caps[PDO4].PeakI = PEAK_CURRENT_0,
    [PORT0].src_caps[PDO4].MinV = PDO_VOLT(20000),
    [PORT0].src_caps[PDO4].MaxV = 0,   /* N/A */
    [PORT0].src_caps[PDO4].MaxI = PDO_CURR(2250),
    [PORT0].src_caps[PDO4].MaxPower = 0, /* N/A */

    [PORT0].src_caps[PDO5].SupplyType = SUPPLY_TYPE_PROGRAMMABLE,
    [PORT0].src_caps[PDO5].PeakI = PEAK_CURRENT_0,
    [PORT0].src_caps[PDO5].MinV = APDO_VOLT(3000),
    [PORT0].src_caps[PDO5].MaxV = APDO_VOLT(5900),
    [PORT0].src_caps[PDO5].MaxI = APDO_CURR(5000),
    [PORT0].src_caps[PDO5].MaxPower = 0, /* N/A */

    [PORT0].src_caps[PDO6].SupplyType = SUPPLY_TYPE_PROGRAMMABLE,
    [PORT0].src_caps[PDO6].PeakI = PEAK_CURRENT_0,
    [PORT0].src_caps[PDO6].MinV = APDO_VOLT(3000),
    [PORT0].src_caps[PDO6].MaxV = APDO_VOLT(11000),
    [PORT0].src_caps[PDO6].MaxI = APDO_CURR(3000),
    [PORT0].src_caps[PDO6].MaxPower = 0, /* N/A */

    [PORT0].src_caps[PDO7].SupplyType = SUPPLY_TYPE_PROGRAMMABLE,
    [PORT0].src_caps[PDO7].PeakI = PEAK_CURRENT_0,
    [PORT0].src_caps[PDO7].MinV = APDO_VOLT(3000),
    [PORT0].src_caps[PDO7].MaxV = APDO_VOLT(20000),
    [PORT0].src_caps[PDO7].MaxI = APDO_CURR(2250),
    [PORT0].src_caps[PDO7].MaxPower = 0, /* N/A */

    [PORT0].src_settling_time_ms = 50,

    [PORT0].num_snk_pdos = 4,
    /* vSafe5V Sink PDO */
    [PORT0].snk_caps[PDO1].SupplyType = SUPPLY_TYPE_FIXED, 
    [PORT0].snk_caps[PDO1].MinV = PDO_VOLT(5000),
    [PORT0].snk_caps[PDO1].MaxV = 0, /* N/A */
    [PORT0].snk_caps[PDO1].MaxOperatingCurrent = PDO_CURR(3000),
    [PORT0].snk_caps[PDO1].MinOperatingCurrent = PDO_CURR(100),
    [PORT0].snk_caps[PDO1].OperationalCurrent = PDO_CURR(3000),
    [PORT0].snk_caps[PDO1].MaxOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO1].MinOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO1].OperationalPower = 0,  /* N/A */
    /* Sink PDO #2 */
    [PORT0].snk_caps[PDO2].SupplyType = SUPPLY_TYPE_FIXED, 
    [PORT0].snk_caps[PDO2].MinV = PDO_VOLT(9000),
    [PORT0].snk_caps[PDO2].MaxV = 0, /* N/A */
    [PORT0].snk_caps[PDO2].MaxOperatingCurrent = PDO_CURR(3000),
    [PORT0].snk_caps[PDO2].MinOperatingCurrent = PDO_CURR(100),
    [PORT0].snk_caps[PDO2].OperationalCurrent = PDO_CURR(3000),
    [PORT0].snk_caps[PDO2].MaxOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO2].MinOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO2].OperationalPower = 0,  /* N/A */

    /* Sink PDO #3 */
    [PORT0].snk_caps[PDO3].SupplyType = SUPPLY_TYPE_FIXED,
    [PORT0].snk_caps[PDO3].MinV = PDO_VOLT(15000),
    [PORT0].snk_caps[PDO3].MaxV = 0, /* N/A */
    [PORT0].snk_caps[PDO3].MaxOperatingCurrent = PDO_CURR(3000),
    [PORT0].snk_caps[PDO3].MinOperatingCurrent = PDO_CURR(100),
    [PORT0].snk_caps[PDO3].OperationalCurrent = PDO_CURR(3000),
    [PORT0].snk_caps[PDO3].MaxOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO3].MinOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO3].OperationalPower = 0,  /* N/A */

    /* Sink PDO #3 */
    [PORT0].snk_caps[PDO4].SupplyType = SUPPLY_TYPE_FIXED,
    [PORT0].snk_caps[PDO4].MinV = PDO_VOLT(20000),
    [PORT0].snk_caps[PDO4].MaxV = 0, /* N/A */
    [PORT0].snk_caps[PDO4].MaxOperatingCurrent = PDO_CURR(2000),
    [PORT0].snk_caps[PDO4].MinOperatingCurrent = PDO_CURR(100),
    [PORT0].snk_caps[PDO4].OperationalCurrent = PDO_CURR(2000),
    [PORT0].snk_caps[PDO4].MaxOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO4].MinOperatingPower = 0, /* N/A */
    [PORT0].snk_caps[PDO4].OperationalPower = 0,  /* N/A */

    [PORT0].higher_capability = false,
    [PORT0].giveback_flag = false,
    [PORT0].no_usb_suspend = true,
    [PORT0].fast_role_swap_support = FR_SWAP_NOT_SUPPORTED,
    [PORT0].pdo_priority = PRIORITY_VOLTAGE,
	[PORT0].vconn_source_capable = false,

    [PORT0].auto_accept_swap_to_dfp = true,
    [PORT0].auto_accept_swap_to_ufp = true,
    [PORT0].auto_accept_swap_to_source = true,
    [PORT0].auto_accept_swap_to_sink = true,
    [PORT0].auto_accept_vconn_swap = false,

//	[PORT0].attempt_discover_identity = true,
    [PORT0].multi_function_preferred = false,

    [PORT0].ufp_alt_mode_entry_timeout_enable = false,
    [PORT0].cert_stat_vdo = 0,
    [PORT0].product_vdo = ((uint32_t)USB_PID << 16) | USB_BCD_DEVICE,
    [PORT0].num_product_type_vdos = 0,
    [PORT0].product_type_vdos[0] = 0,
    [PORT0].product_type_vdos[1] = 0,
    [PORT0].product_type_vdos[2] = 0,

#ifdef DP_ALT_MODE_HOST /* DP alt-mode host */

    [PORT0].id_header_vdo = (0xD6000000 +  USB_VID), /* Alt-mode controller */
    [PORT0].num_svids = 1,
    [PORT0].svids[0] = 0xFF01, /* 0xFF01 = DisplayPort */
    [PORT0].svids[1] = 0,
    [PORT0].svids[2] = 0,
    [PORT0].modes[0] = 0x1C46,  /* 0x1C46 = DFP_D, DP_v1.3, receptacle, USB 2 signaling may be required, DFP_D pin assignment C, D, E */
    [PORT0].modes[1] = 0,
    [PORT0].modes[2] = 0,

#elif defined HDMI_ALT_MODE_HOST /* HDMI alt-mode host */

    [PORT0].id_header_vdo = (0xD6000000 + USB_VID), /* Alt-mode controller */
    [PORT0].num_svids = 1,
    [PORT0].svids[0] = 0xFF04, /* 0xFF04 = HDMI */
    [PORT0].svids[1] = 0,
    [PORT0].svids[2] = 0,
    [PORT0].modes[0] = 0,
    [PORT0].modes[1] = 0,
    [PORT0].modes[2] = 0,

#else /* No Alt-mode support */

    [PORT0].id_header_vdo = (0xD1000000 + USB_VID),
    [PORT0].num_svids = 0,
    [PORT0].svids[0] = 0, 
    [PORT0].svids[1] = 0,
    [PORT0].svids[2] = 0,
    [PORT0].modes[0] = 0,
    [PORT0].modes[1] = 0,
    [PORT0].modes[2] = 0,

#endif
};

#endif

const struct pb_config pb_config = {
    .cell_num = 3,
    .charge_current = 4000,
    .charge_voltage = 4150,
    .bat_rsense = 10,
    .bus_rsense = 10,
    .loadA_over_current = 2400,
    .loadA_min_current = 50,
    .tape_current = 200,
    .batt_uv_threshold = 3100,
    .batt_shutdown_volt = 3000,
};


power_bank_t pb;

struct tusb422_timer_t periodic_timer;

void periodic_timer_start()
{
   timer_start(&periodic_timer, 100, periodic_task);
}


void pb_pd_explicit_contract_cb(bool asSrc, bool connect)
{
    if(connect) {
        pb.pd_connected = true;
        periodic_timer_start();
    }
    else {

        pb.pd_connected = false;
        pb.is_charge_done = false;
    }

    if (connect) {
        bq2571x_init();
    }
    if (asSrc)
        bq2571x_set_adc_current_sel(false);
    else
        bq2571x_set_adc_current_sel(true);

}

void pb_pd_cc_cb(bool attached, bool asSRC)
{
    pb.cc_attached = attached;

    if (pb.cc_attached)
        timer_start(&periodic_timer, 100, periodic_task);
    else{
        bq2571x_disable_otg();
        pb.otg_active = false;
    }

    if (asSRC){
        pb_src_vbus_enable_cb(5000, 3000);
    }
}


void pb_snk_vbus_enable_cb()
{

    //TODO: setup bq2571x vindpm/iidpm threshold
    int vindpm_th = pb.snk_vbus_mv;
    int ma = pb.snk_vbus_ma;

    if (vindpm_th >= 9000)
        vindpm_th -= 1280;
    else
        vindpm_th -= 600;
    if (vindpm_th < 0)
        vindpm_th = 4500;

    bq2571x_set_input_voltage_limit((uint16_t)vindpm_th);
    bq2571x_set_input_current_limit(ma);

    bq2571x_set_charge_voltage(pb_config.charge_voltage * pb_config.cell_num);
    bq2571x_set_charge_current(pb_config.charge_current);

    bq2571x_enable_charge();
    bq2571x_disable_otg();


}

void pb_src_vbus_enable_cb(int mv, int ma)
{

    bq2571x_set_otg_voltage(mv);
    bq2571x_set_otg_current(ma);

    GPIO_setOutputHighOnPin(OTG_EN_PORT,OTG_EN_PIN);
    bq2571x_enable_otg();
}

void pb_led_timer_cb()
{
#if 0
    if (!pb.otg_active && pb.is_undervoltage) {
        pb.is_undervoltage = false;
        pb.loadA_pres = false;
    }
#endif
    if (pb.is_overloaded)
        pb.is_overloaded = false;

}


static void pb_check_batt_uv()
{
    if (pb.is_charging) {
        pb.is_undervoltage = false;
        return;
    }

    if (pb.bat_volt < pb_config.batt_shutdown_volt * pb_config.cell_num) {
        if (pb.otg_active) {
            bq2571x_disable_otg();
            pb.otg_active = false;
            pb.cc_attached = pb.pd_connected = pb.ac_pres = 0;
        }
        if (pb.loadA_pres) {
            en_out_disable();
            pb.loadA_pres = false;
        }
    }
    else if (pb.bat_volt < pb_config.batt_uv_threshold * pb_config.cell_num)
        pb.is_undervoltage = true;
    else if (pb.bat_volt > pb_config.batt_uv_threshold * pb_config.cell_num + 100)
        pb.is_undervoltage = false;
}

static void pb_check_loadA()
{
    static unsigned char load_remove_timer;

    if (!pb.loadA_pres) {
        load_remove_timer = 0;
        return;
    }

    if(pb.ioA_curr > pb_config.loadA_over_current){//over load
         pb.is_overloaded = 1;
         load_remove_timer = 0;
     }
     else {
         pb.is_overloaded = 0;
         en_out_enable();
         if(pb.ioA_curr <= pb_config.loadA_min_current){ //FULL or detached
             if(++load_remove_timer > 60) {
                 en_out_disable();
                 load_remove_timer = 0;
                 pb.loadA_pres = false;
             }
         }
         else {
             load_remove_timer = 0;
         }
     }

}


#define     TAPE_CURRENT    200
#define     TAPE_TIME       5
// it is called every second
static void pb_check_charge_done()
{
    static uint16_t tape_timer;

    if (pb.is_charging && !pb.is_charge_done
            && pb.bat_volt > (pb_config.charge_voltage - 50) * pb_config.cell_num
            && pb.bat_curr < pb_config.tape_current) {
        tape_timer++;
        if (tape_timer > TAPE_TIME) {
            pb.is_charge_done = true;
            bq2571x_disable_charge();
        }
    } else if (pb.ac_pres && !pb.otg_active && pb.is_charge_done
            && pb.bat_volt < (pb_config.charge_voltage - 100) * pb_config.cell_num){
        //recharge
        pb.is_charge_done = false;
        bq2571x_enable_charge();
        tape_timer = 0;
    } else
        tape_timer = 0;
}

static void pb_update_status()
{
    bq2571x_get_ac_present(&pb.ac_pres);
    bq2571x_get_charging_status(&pb.is_charging);
    bq2571x_get_fault_status(&pb.charge_fault);
    bq2571x_get_otg_mode(&pb.otg_active);
    pb.otg_fault = pb.charge_fault & 0x03;//OTG_OVP or OTG_UVP
    pb.charge_fault &= ~0x03;

#ifndef USE_MSP430_ADC_IBAT_IBUS
    bq2571x_read_chg_current(&pb.bat_curr);
#endif

    adc_update_results();
    pb.rsoc = rsoc_update(pb.bat_volt);

    pb_check_charge_done();


}

static void led_task()
{
    if (pb.led_event_ongoing){
        led_display();
        return;
    }

    if (pb.charge_fault || pb.is_overloaded || pb.otg_fault){
        pb.led_ctrl = LED_ON | LED_BLINK_FAULT;
        pb.key_pressed = false;//ignore key press LED;
    }
    else if (pb.is_undervoltage ) {
        pb.led_ctrl = LED_ON | LED_BLINK_UV;
        pb.key_pressed = false;//ignore key press LED;
    }
    else if(pb.is_charging){
        pb.led_ctrl = LED_ON | LED_BLINK_RSOC; // display led and blink last one
        pb.key_pressed = false;//ignore key press LED on during charging;
    }
    else if(pb.is_charge_done){
        pb.led_ctrl = LED_ON;// display led but  not blink
    }
    else if(pb.key_pressed){
        pb.key_pressed = false;
        pb.led_ctrl = LED_ON | LED_ON_3S;
    }
    else {
        pb.led_ctrl = 0;
    }

    led_display();
}

static void periodic_task(unsigned int port)
{
    static unsigned int count;

    count++;

    if (count % 10 == 0) {
        bq2571x_reg_dump();
    }

    if (count % 5 == 0) {
        pb_update_status();
        pb_check_batt_uv();
        pb_check_loadA();

        led_task();
    }

    if (pb.cc_attached || pb.pd_connected || pb.ac_pres || pb.loadA_pres || pb.led_event_ongoing)
        periodic_timer_start();
    else {
        // force LED off
        pb.led_ctrl = 0;
        led_display();
    }
}


//*****************************************************************************
//
//! Main Function
//!
//! Device Policy Manager
//!
//! \return none
//
// *****************************************************************************
int main(void)
{
    unsigned int port = NUM_TCPC_DEVICES - 1;

    // Set alert for all ports to handle any pending IRQ.
    do
    {
        g_alert_event |= (1 << port);
    } while (port--);

    // Initialize default CLK and GPIO settings.
    tcpm_hal_init();    

    led_init();

  //  uart_init();

    adc_init();

    timer_init();
    __enable_interrupt();

//   usb_pd_print_version();

    tcpm_init(typec_config);
    usb_pd_init(pd_config);

//    tcpm_hal_led_high(LED_GREEN);

    bq2571x_init();
#ifdef USE_MSP430_ADC_IBAT_IBUS
     bq2571x_get_ibus_ibat_gain(&pb.ibus_gain, &pb.ibat_gain);
#endif

    while (1)
    {
        port = NUM_TCPC_DEVICES - 1;

        do
        {
            /* 
             * To avoid I2C reentrancy issues, the alert and timer handling is
             * all done within this loop immediately followed by the state
             * machines.
             */ 
            if (g_alert_event & (1 << port))
            {
                g_alert_event &= ~(1 << port);

                /* Process alerts until ALERT# pin is no longer asserted */
                do
                {
                    tcpm_alert_event(port);
                    tcpm_connection_state_machine(port);
                    usb_pd_pe_state_machine(port);
                } while (tcpm_is_alert_asserted(port));
            }

            if (g_timer_expired & (1 << port))
            {
                g_timer_expired &= ~(1 << port);

                g_timer_callback[port](port);

                tcpm_connection_state_machine(port);
                usb_pd_pe_state_machine(port);
            }
        } while (port--);

#if DEBUG_LEVEL >= 1
        if (bRegDump)
        {
            bRegDump = false;
            tcpm_register_dump(0);
        }

        if (bErrorRecovery)
        {
            bErrorRecovery = false;
            tcpm_execute_error_recovery(0);
        }

        if (bTryRoleSwap)
        {
            bTryRoleSwap = false;
            tcpm_try_role_swap(0);
        }

        if (bSendHardReset)
        {
            bSendHardReset = false;
            usb_pd_policy_manager_request(0, PD_POLICY_MNGR_REQ_HARD_RESET);
        }

        if (bPRSwap)
        {
            bPRSwap = false;
            usb_pd_policy_manager_request(0, PD_POLICY_MNGR_REQ_PR_SWAP);
        }

        if (bDRSwap)
        {
            bDRSwap = false;
            usb_pd_policy_manager_request(0, PD_POLICY_MNGR_REQ_DR_SWAP);
        }

        if (bVCONNSwap)
        {
            bVCONNSwap = false;
            usb_pd_policy_manager_request(0, PD_POLICY_MNGR_REQ_VCONN_SWAP);
        }

#endif

        __disable_interrupt();

        if (!(g_alert_event || g_timer_expired))
        {
            // Enter low power mode.  
#ifdef __MSP432P401R__
            __sleep();
            __enable_irq();
            __ISB();
#else
            _BIS_SR(LPM3_bits + GIE);
#endif
        }
        else
        {
          __enable_interrupt();
        }
    }

//    return 0;   /* unreachable */
}
