/*
 * TUSB422 Power Delivery
 *
 * Author: Brian Quach <brian.quach@ti.com>
 *
 * Copyright (C) 2016 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __TIMER_H__
#define __TIMER_H__

#include "tusb422_common.h"
#include <stdint.h>
#include <stdbool.h>

typedef void (*timer_callback_t)(unsigned int);


//*****************************************************************************
//
//! \brief Initializes the timer subsystem. 
//
// ****************************************************************************
void timer_init(void);


//*****************************************************************************
//
//! \brief Starts a timer.
//!
//! \param timer        pointer to timer struct.              
//! \param timeout_ms   time in milliseconds              
//! \param function     pointer to function to call upon expiration  
//!                
//! \return timer list index or -1 if no timers available
//
// ****************************************************************************
//int timer_start(struct tusb422_timer_t *timer, unsigned int timeout_ms, void (*function)(unsigned int));   


//*****************************************************************************
//
//! \brief Cancels a timer.
//!
//! \param timer pointer to timer            
//
// ****************************************************************************
//void timer_cancel(struct tusb422_timer_t *timer);


void msleep(uint16_t msecs);



#endif //__TIMER_H__


