/*
 * led.c
 *
 *  Created on: Aug 25, 2015
 *      Author: a0220433
 */
#include <stdint.h>
#include <stdbool.h>
#include "msp430.h"
#include "gpio.h"
#include "pb.h"
extern power_bank_t pb;

static unsigned char rsoc_to_led()
{
	switch(pb.rsoc){
	case 0:
		return 0x00;
	case 25:
		return 0x01;
	case 50:
		return 0x03;
	case 75:
		return 0x07;
	case 100:
		return 0x0F;
	default:
		return 0x03;
	}

}

extern void pb_led_timer_cb();

void led_display()
{
    unsigned char led_mask; //control which LEDs will be on or blink
    unsigned char led;

    static unsigned char last_led = 0;
    static unsigned char last_led_mask = 0;
    static unsigned char led_timer = 0;
    static unsigned char led_timer_mask;
    static bool led_blink_fault;
    static bool led_blink_uv = false;


    led_mask = rsoc_to_led();

    if(led_timer){
        pb.led_ctrl |= LED_ON; //avoid cleared outside
        led_mask = led_timer_mask;
        led_timer--;
        if(!led_timer){
            pb.led_ctrl = 0; // disable display when time out
            pb.led_event_ongoing = 0;
            pb_led_timer_cb();
            led_blink_fault = false;
            led_blink_uv = false;
        }
    }
    else if(pb.led_ctrl & LED_BLINK_FAULT){
        led_blink_fault = true;
        pb.led_event_ongoing = true;
        led_timer_mask = 0x0F;
        led_timer = 6;
    }
    else if (pb.led_ctrl & LED_BLINK_UV){
        led_blink_uv = true;
        led_mask = 0x01;
 //       pb.led_event_ongoing = true;
 //       led_timer = 6;
    }
    else if(pb.led_ctrl & LED_ON_3S){
        led_blink_fault = false;
        led_blink_uv = false;
        led_timer = 6;
        led_timer_mask = led_mask;
        pb.led_event_ongoing = true;
    }

    if(led_mask == last_led_mask){
        if(led_blink_fault){
            led = last_led ^ 0x0F;
            last_led = led;
        } else if (led_blink_uv){
            led = last_led ^ 0x01;
            last_led = led;
        }
        else if(pb.led_ctrl & LED_BLINK_RSOC){
            if (led_mask == 0)
                led = last_led ^ 1;
            else
                led = last_led ^ ((led_mask + 1) >> 1);
            last_led = led;
        } else{
            led = led_mask;
        }
    }
    else{
        last_led_mask = led_mask;
        led = led_mask;
        last_led = led;
    }

    if(!(pb.led_ctrl & LED_ON)){
        led = 0;
        last_led = led;
        last_led_mask = 0;
        led_blink_uv = 0;
        led_blink_fault = 0;
    }

    led <<= 1;

    if (pb.otg_active) {
        led |= 1;
    }

   	PJOUT &= ~0x1F;
	PJOUT |= led & 0x1F;

}

void led_init()
{
    uint8_t led;
    GPIO_setAsOutputPin(GPIO_PORT_PJ,0x1F);
    pb.led_ctrl = 0;

    led = 0;
    PJOUT &= ~0x1F;
    PJOUT |= led & 0x1F;

}
